Package: igblastr
Title: User-friendly R Wrapper to IgBLAST
Description: The igblastr package provides functions to conveniently install
	and use a local IgBLAST installation from within R. The package also
	includes a set of built-in IgBLAST-compatible germline databases from
	OGRDB, the AIRR Community’s Open Germline Receptor Database, for
	various organisms. It provides functions to create additional
	IgBLAST-compatible germline databases using reference sequences
	retrieved from IMGT/V-QUEST or local FASTA files supplied by the user.
	When possible, annotations for the V and J alleles in a new germline
	database are automatically generated and added to the database, so
	they can be used as replacements for the internal and auxiliary data
	provided by IgBLAST.
	IgBLAST is described at <https://pubmed.ncbi.nlm.nih.gov/23671333/>.
	IgBLAST web interface: <https://www.ncbi.nlm.nih.gov/igblast/>.
	OGRDB: <https://ogrdb.airr-community.org/>.
	IMGT/V-QUEST download site: <https://www.imgt.org/download/V-QUEST/>.
biocViews: Immunology, Immunogenetics, ImmunoOncology, CellBiology
URL: https://bioconductor.org/packages/igblastr
BugReports: https://github.com/HyrienLab/igblastr/issues
Version: 1.2.15
License: Artistic-2.0
Encoding: UTF-8
Authors@R: c(
	person("Hervé", "Pagès", role=c("aut", "cre"),
	       email="hpages.on.github@gmail.com",
	       comment=c(ORCID="0009-0002-8272-4522")),
	person("Ollivier", "Hyrien", role=c("aut", "fnd"),
	       comment=c(ORCID="0000-0003-1909-2542")),
	person("Kellie", "MacPhee", role="ctb",
	       comment=c(ORCID="0009-0008-0993-4009")),
	person("Michael", "Duff", role="ctb",
	       comment=c(ORCID="0009-0008-4279-0756")),
	person("Jason", "Taylor", role="ctb"))
Depends: R (>= 4.2.0), tibble, Biostrings
Imports: methods, utils, stats, tools, R.utils, curl, httr, xml2,
        rvest, xtable, jsonlite, BiocGenerics, S4Vectors, IRanges,
        GenomeInfoDb
Suggests: GenomicAlignments, parallel, testthat, knitr, rmarkdown,
        BiocStyle, ggplot2, dplyr, scales, ggseqlogo, airr
VignetteBuilder: knitr
Collate: utils.R internet-utils.R long_to_wide_airr.R igdata-IO.R
        translate_codons.R allele2gene.R parse_imgt_fasta_headers.R
        file-utils.R loci-utils.R db-utils.R ndm_data-IO.R
        compute_V_gene_delineations.R auxdata-IO.R compute_auxdata.R
        clean_allele_set.R V_alleles-inspect.R J_alleles-inspect.R
        LATIN_NAMES.R IMGT-utils.R IMGT-c_region-utils.R
        precompiled-igblast-utils.R cache-utils.R get_igblast_root.R
        edit_imgt_file.R igblast_info.R update_live_igdata.R
        load_intdata.R load_auxdata.R intdata-misc.R auxdata-misc.R
        install_igblast.R make_blastdbs.R create_region_db.R
        create_germline_db.R create_c_region_db.R reset_germline_dbs.R
        list_germline_dbs.R use_germline_db.R reset_c_region_dbs.R
        list_c_region_dbs.R use_c_region_db.R
        install_custom_germline_db.R OGRDB-utils.R OGRDB-API.R
        download_OGRDB_germline_sequences.R
        download_OGRDB_germline_json.R install_OGRDB_germline_db.R
        download_IMGT_germline_sequences.R install_IMGT_germline_db.R
        combine_germline_dbs.R augment_germline_db.R
        prepare_igblastn_cmdline_args.R read_igblastn_fmt7_output.R
        read_igblastn_AIRR_output.R igblastn.R igbrowser.R
        percent_mutation.R summarizeMismatches.R OAS-utils.R zzz.R
git_url: https://git.bioconductor.org/packages/igblastr
git_branch: RELEASE_3_23
git_last_commit: e759305
git_last_commit_date: 2026-07-17
Repository: Bioconductor 3.23
Date/Publication: 2026-07-21
NeedsCompilation: no
Packaged: 2026-07-21 22:25:36 UTC; biocbuild
Author: Hervé Pagès [aut, cre] (ORCID: <https://orcid.org/0009-0002-8272-4522>),
  Ollivier Hyrien [aut, fnd] (ORCID:
    <https://orcid.org/0000-0003-1909-2542>),
  Kellie MacPhee [ctb] (ORCID: <https://orcid.org/0009-0008-0993-4009>),
  Michael Duff [ctb] (ORCID: <https://orcid.org/0009-0008-4279-0756>),
  Jason Taylor [ctb]
Maintainer: Hervé Pagès <hpages.on.github@gmail.com>
