This is the extdata/ folder in the igblastr package.

This folder contains various data files used in the man page examples and/or
unit tests of the igblastr package.

Content
=======

- README.txt: This file.

- BCR/: See README.txt in BCR/ subfolder.

- constant_regions/: See README.txt in constant_regions/ subfolder.

- germline_sets/: See README.txt in germline_sets/ subfolder.

- igdata_store/: See README.txt in igdata_store/ subfolder.

- novel_germline_alleles/: See README.txt in novel_germline_alleles/
  subfolder.

- TCR/: See README.txt in TCR/ subfolder.

