This is the extdata/germline_sets/ folder in the igblastr package.

This folder contains germline sequences (in FASTA format) downloaded from
various places (e.g. AIRR-community/OGRDB) for various organisms.

Content
=======

- README.txt: This file.

- OGRDB/: See README.txt in OGRDB/ subfolder.

