This is the extdata/TCR/ folder in the igblastr package.

This folder contains TCR sequencing datasets used in the man page examples
and/or unit tests of the igblastr package.

Content
=======

- README.txt: This file.

- SRR11341217.fasta.gz: 10,875 human beta chain TCR transcripts running from 5'
  of reverse transcription reaction to beginning of constant region, produced
  by merging R1 and R2 reads. This dataset was downloaded from the Short Read
  Archive on October 7, 2025:
    o Short Read Archive accession number: SRX7944361
    o URL: https://www.ncbi.nlm.nih.gov/sra/SRX7944361
    o Download command: curl https://trace.ncbi.nlm.nih.gov/Traces/sra-reads-be/fasta?acc=SRR11341217 >SRR11341217.fasta.gz

