This is the extdata/constant_regions/ folder in the igblastr package.

This folder contains the C-regions (in FASTA format) downloaded from various
places (well, only IMGT at the moment) for various organisms.

Content
=======

- README.txt: This file.

- IMGT/: See README.txt in IMGT/ subfolder.

