This is the extdata/novel_germline_alleles/ folder in the igblastr package.

This folder contains novel gene allele sequences in FASTA format.

Content
=======

- README.txt: This file.

- fake_human_V_alleles.fasta: 3 made-up novel alleles for human.
  See fake_human_V_alleles.fasta.notes for the details.

- fake_human_D_alleles.fasta: 5 made-up novel alleles for human.
  See fake_human_D_alleles.fasta.notes for the details.

