This is the extdata/germline_sets/OGRDB/ folder in the igblastr package.

This folder contains human and mouse germline sequences (in FASTA format)
downloaded from AIRR-community/OGRDB.

Content
=======

- README.txt: This file.

- human/: See README.txt in human/ subfolder.

- mouse/: See README.txt in mouse/ subfolder.

- rhesus_monkey/: See README.txt in rhesus_monkey/ subfolder.

