This is the extdata/germline_sets/OGRDB/rhesus_monkey/ folder in the
igblastr package.

This folder contains germline sequences (in FASTA format) downloaded from
AIRR-community/OGRDB for rhesus monkey.

Content
=======

- README.txt: This file.

- 202601/: See README.md in 202601/ subfolder.

- 202602/: See README.md in 202602/ subfolder.

- download_rhesus_monkey_germline_sequences.R: The code used to generate all
           the FASTA files in the various subfolders of this folder.

