This is the extdata/germline_sets/OGRDB/human/ folder in the igblastr
package.

This folder contains germline sequences (in FASTA format) downloaded from
AIRR-community/OGRDB for human.

Content
=======

- README.txt: This file.

- 202309/: See README.md in 202309/ subfolder.

- 202401/: See README.md in 202401/ subfolder.

- 202410/: See README.md in 202410/ subfolder.

- 202605/: See README.md in 202605/ subfolder.

- diffs/: A collection of R scripts that show what has changed between
          versions of the _OGRDB.human.IGH+IGK+IGL.* germline dbs.

- download_human_germline_sequences.R: The code used to generate all
           the FASTA files in the various subfolders of this folder.

