This is the extdata/germline_sets/OGRDB/rainbow_trout/ folder in the
igblastr package.

This folder contains germline sequences (in FASTA format) downloaded from
AIRR-community/OGRDB for rainbow trout (Oncorhynchus mykiss).

Content
=======

- README.txt: This file.

- 202606/: See README.md in 202606/ subfolder.

- download_rainbow_trout_germline_sequences.R: The code used to generate all
           the FASTA files in the various subfolders of this folder.

