This is the extdata/germline_sets/OGRDB/mouse/ folder in the igblastr
package.

This folder contains germline sequences (in FASTA format) downloaded from
AIRR-community/OGRDB for various mouse strains.

Content
=======

- README.txt:  This file.

- CAST_EiJ/:   Germline sequences for mouse strain CAST/EiJ. See README.md
               in CAST_EiJ/ subfolder for more information.

- LEWES_EiJ/:  Germline sequences for mouse strain LEWES/EiJ. See README.md
               in LEWES_EiJ/ subfolder for more information.

- MSM_MsJ/:    Germline sequences for mouse strain MSM/MsJ. See README.md
               in MSM_MsJ/ subfolder for more information.

- NOD_ShiLtJ/: Germline sequences for mouse strain NOD/ShiLtJ. See README.md
               in NOD_ShiLtJ/ subfolder for more information.

- PWD_PhJ/:    Germline sequences for mouse strain PWD/PhJ. See README.md
               in PWD_PhJ/ subfolder for more information.

- download_mouse_germline_sequences.R: The code used to generate all
           the FASTA files in the strain-specific subfolders of this folder.

