\name{summarizeMismatches}

\alias{summarizeMismatches}
\alias{tabulate_mismatches}
\alias{tabulate_insertions}
\alias{tabulate_deletions}

\title{Summarize mismatches and indels between query and germline sequences}

\description{
  TODO
}

\keyword{utilities}
